import React from 'react';
import {
  readInSchemaFromBackend,
  useAuth,
  useAuthorizationCache,
  useDatabaseAPI,
  useQueryAPI,
  useQuerybuilderGraph,
  useQuerybuilderHash,
  useSchemaAPI,
  useSessionCache,
} from '@graphpolaris/shared/lib/data-access';
import { WebSocketHandler } from '@graphpolaris/shared/lib/data-access/socket';
import Broker from '@graphpolaris/shared/lib/data-access/socket/broker';
import {
  assignNewGraphQueryResult,
  useAppDispatch,
  useConfig,
  useML,
  useMLEnabledHash,
  useQuerybuilderSettings,
} from '@graphpolaris/shared/lib/data-access/store';
import {
  GraphQueryResultFromBackend,
  GraphQueryResultFromBackendPayload,
  resetGraphQueryResults,
} from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';
import { Query2BackendQuery, QueryBuilder, QueryMultiGraph } from '@graphpolaris/shared/lib/querybuilder';
import { Schema } from '@graphpolaris/shared/lib/schema/panel';
import { useCallback, useEffect, useRef, useState } from 'react';
import { Navbar } from '../components/navbar/navbar';
import { VisualizationPanel } from './panels/Visualization';
import styles from './app.module.scss';
import { logout } from '@graphpolaris/shared/lib/data-access/store/authSlice';
import { SchemaFromBackend } from '@graphpolaris/shared/lib/schema';
import { LinkPredictionInstance, setMLResult, allMLTypes } from '@graphpolaris/shared/lib/data-access/store/mlSlice';
import { Resizable } from '@graphpolaris/shared/lib/components/Resizable';
import { DashboardAlerts } from '@graphpolaris/shared/lib/data-access/authorization/dashboardAlerts';
import { addError } from '@graphpolaris/shared/lib/data-access/store/configSlice';

export interface App {}

export function App(props: App) {
  const { login } = useAuth();
  const auth = useAuthorizationCache();
  const api = useDatabaseAPI();
  const api_schema = useSchemaAPI();
  const api_query = useQueryAPI();
  const dispatch = useAppDispatch();
  const session = useSessionCache();
  const query = useQuerybuilderGraph() as QueryMultiGraph;
  const queryHash = useQuerybuilderHash();
  const ws = useRef(new WebSocketHandler(import.meta.env.BACKEND_WSS_URL));
  const [authCheck, setAuthCheck] = useState(false);
  const ml = useML();
  const mlHash = useMLEnabledHash();
  const config = useConfig();
  const queryBuilderSettings = useQuerybuilderSettings();

  // for testing purposes
  // useEffect(() => {
  //   console.info('Authentification changed', auth)
  // }, [auth]);

  useEffect(() => {
    // Default
    Broker.instance().subscribe((data: SchemaFromBackend) => dispatch(readInSchemaFromBackend(data)), 'schema_result');
    Broker.instance().subscribe((data: GraphQueryResultFromBackendPayload) => dispatch(assignNewGraphQueryResult(data)), 'query_result');
    allMLTypes.forEach((mlType) => {
      Broker.instance().subscribe((data: LinkPredictionInstance[]) => dispatch(setMLResult({ type: mlType, result: data })), mlType);
    });

    login();

    return () => {
      Broker.instance().unSubscribeAll('schema_result');
      Broker.instance().unSubscribeAll('query_result');
      allMLTypes.forEach((mlType) => {
        Broker.instance().unSubscribeAll(mlType);
      });
    };
  }, []);

  useEffect(() => {
    // New active database
    if (auth.jwt && session.currentDatabase) {
      ws.current.useAuth(auth).connect(() => {
        api_schema.RequestSchema(session.currentDatabase);
      });
    }
  }, [session.currentDatabase]);

  useEffect(() => {
    // Newly (un)authorized
    if (auth.authorized) {
      console.debug('App is authorized; Getting Databases', auth.authorized);
      api.GetAllDatabases({ updateSessionCache: true }).catch((e) => {
        dispatch(addError(e.message));
      });
      setAuthCheck(true);
    } else {
      // dispatch(logout());
    }
  }, [auth]);

  const runQuery = () => {
    if (session?.currentDatabase && query) {
      if (query.nodes.length === 0) {
        dispatch(resetGraphQueryResults());
      } else {
        api_query.execute(Query2BackendQuery(session.currentDatabase, query, queryBuilderSettings, ml));
      }
    }
  };

  useEffect(() => {
    runQuery();
  }, [queryHash, mlHash, queryBuilderSettings]);

  return (
    <div className="h-screen w-screen overflow-clip">
      <DashboardAlerts />
      <div className={'h-screen w-screen ' + (!auth.authorized ? 'blur-sm pointer-events-none ' : '')}>
        <div className="flex flex-col h-screen max-h-screen relative">
          <aside className="h-[4rem]">
            <Navbar />
          </aside>
          <main className="flex w-screen h-[calc(100%-4.2rem)]">
            <Resizable divisorSize={3} horizontal={true} defaultProportion={0.33}>
              <div className="h-full w-full panel">
                <Schema auth={authCheck} />
              </div>
              <div className="h-full w-full">
                <Resizable divisorSize={3} horizontal={false}>
                  <div className="w-full h-full panel">
                    <VisualizationPanel />
                  </div>
                  <div className="w-full h-full panel">
                    <QueryBuilder
                      onRunQuery={() => {
                        console.log('Run Query');
                        runQuery();
                      }}
                    />
                  </div>
                </Resizable>
              </div>
            </Resizable>
          </main>
        </div>
      </div>
      {/*<GridLayout*/}
      {/*  className="layout"*/}
      {/*  cols={10}*/}
      {/*  rowHeight={30}*/}
      {/*  width={window.innerWidth}*/}
      {/*>*/}
      {/*  <div*/}
      {/*    key="schema-panel"*/}
      {/*    data-grid={{ x: 0, y: 0, w: 3, h: 30, maxW: 5, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <Panel content="Schema Panel" color="red">*/}
      {/*      <Schema />*/}
      {/*    </Panel>*/}
      {/*  </div>*/}
      {/*  <div*/}
      {/*    key="query-panel"*/}
      {/*    data-grid={{ x: 3, y: 20, w: 5, h: 10, maxH: 20, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <Panel content="Query Panel" color="blue">*/}
      {/*      <QueryBuilder />*/}
      {/*    </Panel>*/}
      {/*  </div>*/}
      {/*  <div*/}
      {/*    key="visualisation-panel"*/}
      {/*    data-grid={{ x: 3, y: 0, w: 7, h: 20, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <VisualizationPanel />*/}
      {/*  </div>*/}
      {/*  <div*/}
      {/*    key="history-panel"*/}
      {/*    data-grid={{ x: 8, y: 20, w: 2, h: 10, isDraggable: false }}*/}
      {/*  >*/}
      {/*    <Panel content="History Channel" color="purple"></Panel>*/}
      {/*  </div>*/}
      {/*</GridLayout>*/}
    </div>
  );
}

export default App;
