/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useEffect, useState } from 'react';
import { AccountCircle } from '@mui/icons-material';
import logo from './logogp.png';
import logo_white from './logogpwhite.png';

import { updateCurrentDatabase, updateDatabaseList } from '@graphpolaris/shared/lib/data-access/store/sessionSlice';
import {
  AddDatabaseRequest,
  useAppDispatch,
  useAuthorizationCache,
  useDatabaseAPI,
  useSchemaAPI,
  useSessionCache,
  useVisualizationCache,
} from '@graphpolaris/shared/lib/data-access';
import { DatabaseMenu } from './databasemenu';
import { NewDatabaseForm } from './AddDatabaseForm/newdatabaseform';
import { addError } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { SearchBar } from './search/SearchBar';

/** NavbarComponentProps is an interface containing the NavbarViewModel. */
import { setVisual } from '@graphpolaris/shared/lib/data-access/store/visualisationSlice';
export interface NavbarComponentProps {
  // changeColourPalette: () => void; FIXME move to redux
}

/** NavbarComponentState is an interface containing the type of visualizations. */
export interface NavbarComponentState {
  // The anchor for rendering the user data (userID, sessionID, add database, etc) menu
  userMenuAnchor?: Element;
  // The anchor for rendering the menu for selecting a database to use
  selectDatabaseMenuAnchor?: Element;
  deleteDatabaseMenuAnchor?: Element;
  // Determines if the addDatabaseForm will be shown
  showAddDatabaseForm: boolean;
}

export interface NavbarSubComponentState {
  changeDb?: String;
  deleteDb?: String;
}

/** NavbarComponent is the View implementation for Navbar */
export const Navbar = (props: NavbarComponentProps) => {
  const auth = useAuthorizationCache();
  const session = useSessionCache();
  const vis = useVisualizationCache();
  const api = useDatabaseAPI();
  const schemaApi = useSchemaAPI();
  const dispatch = useAppDispatch();

  const [addDatabaseFormOpen, setAddDatabaseFormOpen] = useState(false);
  const [menuOpen, setMenuOpen] = useState(false);
  const [subMenuOpen, setSubMenuOpen] = useState<string | undefined>(undefined);

  const currentLogo = !'dark' ? logo_white : logo; // TODO: support dark mode

  const buildInfo = import.meta.env.GRAPHPOLARIS_VERSION;

  return (
    <div className="w-full h-auto px-5">
      <NewDatabaseForm
        open={addDatabaseFormOpen}
        onClose={() => {
          setAddDatabaseFormOpen(false);
        }}
      />
      <div title="GraphPolaris" className="navbar w-full">
        <a href="https://graphpolaris.com/" className="w-full">
          <img src={currentLogo} />
        </a>
        <SearchBar />
        <div className="dropdown">
          <label tabIndex={0} className="btn s-1">
            Vis
          </label>
          <ul tabIndex={0} className="dropdown-content z-[1] menu p-2 shadow bg-base-100 rounded-box w-52">
            <li
              onClick={() => {
                dispatch(setVisual('NodeLinkVis'));
              }}
            >
              <a>Node Link</a>
            </li>
            <li
              onClick={() => {
                dispatch(setVisual('PaohVis'));
              }}
            >
              <a>PaohVis</a>
            </li>
            <li
              onClick={() => {
                dispatch(setVisual('RawJSONVis'));
              }}
            >
              <a>JSON Structure</a>
            </li>
            {/* <li><a>Semantic Substrates</a></li> */}
          </ul>
        </div>
        <div className="dropdown">
          <label tabIndex={1} className="btn s-1">
            Data
          </label>
          <ul tabIndex={1} className="dropdown-content z-[1] menu p-2 shadow bg-base-100 rounded-box w-52">
            <li>
              <button
                onClick={() => {
                  setAddDatabaseFormOpen(true);
                  setMenuOpen(false);
                  setSubMenuOpen(undefined);
                }}
              >
                Add database
              </button>
            </li>
            <li>
              <button
                onClick={(e) => {
                  e.stopPropagation();
                  setSubMenuOpen(subMenuOpen === 'changeDb' ? undefined : 'changeDb');
                }}
                className={`${session.databases.length === 0 ? 'btn-disabled' : ''} ${subMenuOpen === 'changeDb' ? 'btn-active' : ''}`}
              >
                Change Database
              </button>
              {subMenuOpen === 'changeDb' && (
                <DatabaseMenu
                  onClick={(db) => {
                    if (session.currentDatabase != db) {
                      dispatch(updateCurrentDatabase(db));
                    }
                    setSubMenuOpen(undefined);
                    setMenuOpen(false);
                  }}
                />
              )}
            </li>
            <li>
              <button
                onClick={() => setSubMenuOpen(subMenuOpen === 'deleteDb' ? undefined : 'deleteDb')}
                className={`${session.databases.length === 0 ? 'btn-disabled' : ''} ${subMenuOpen === 'deleteDb' ? 'btn-active' : ''}`}
              >
                Delete Database
              </button>
              {subMenuOpen === 'deleteDb' && (
                <DatabaseMenu
                  onClick={(db) => {
                    if (session.currentDatabase === db) {
                      dispatch(updateCurrentDatabase(''));
                    }
                    api.DeleteDatabase(db);
                    setSubMenuOpen(undefined);
                    setMenuOpen(false);
                  }}
                />
              )}
            </li>
          </ul>
        </div>
        <div className="dropdown">
          <label tabIndex={2} className="btn s-1">
            Adv
          </label>
          <ul tabIndex={2} className="dropdown-content z-[1] menu p-2 shadow bg-base-100 rounded-box w-52"></ul>
        </div>
        <div className="dropdown">
          <label tabIndex={3} className="btn s-1">
            Share
          </label>
          <ul tabIndex={3} className="dropdown-content z-[1] menu p-2 shadow bg-base-100 rounded-box w-52">
            <li>
              <a>Visual</a>
            </li>
            <li>
              <a>Knowledge Base</a>
            </li>
          </ul>
        </div>

        <div className="w-fit">
          <div className="">
            <button
              tabIndex={0}
              className="btn btn-circle btn-ghost hover:bg-gray-200"
              onClick={(event) => {
                setMenuOpen(!menuOpen);
                setSubMenuOpen(undefined);
              }}
            >
              <AccountCircle htmlColor="black" />
            </button>
            {menuOpen && (
              <>
                <div
                  className="z-10 bg-transparent absolute w-screen h-screen top-0 left-0"
                  onClick={() => {
                    setMenuOpen(false);
                    setSubMenuOpen(undefined);
                  }}
                ></div>
                <ul tabIndex={0} className="z-20 dropdown-content menu absolute right-4 p-2 shadow-xl bg-offwhite-100 rounded-box w-52">
                  {auth.authorized ? (
                    <>
                      <div className="menu-title">
                        <h2>user: {auth.userID}</h2>
                        <h3 className="text-xs">session: {auth.sessionID}</h3>
                      </div>
                      <li>
                        <button
                          onClick={() => {
                            setAddDatabaseFormOpen(true);
                            setMenuOpen(false);
                            setSubMenuOpen(undefined);
                            console.log('add database', addDatabaseFormOpen, menuOpen, subMenuOpen);
                          }}
                        >
                          Add database
                        </button>
                      </li>
                      <li>
                        <button
                          onClick={(e) => {
                            e.stopPropagation();
                            setSubMenuOpen(subMenuOpen === 'changeDb' ? undefined : 'changeDb');
                          }}
                          className={`${session.databases.length === 0 ? 'btn-disabled text-gray-300' : ''} ${
                            subMenuOpen === 'changeDb' ? 'btn-active' : ''
                          }`}
                        >
                          Change Database
                        </button>
                        {subMenuOpen === 'changeDb' && (
                          <DatabaseMenu
                            onClick={(db) => {
                              if (session.currentDatabase != db) {
                                dispatch(updateCurrentDatabase(db));
                              }
                              setSubMenuOpen(undefined);
                              setMenuOpen(false);
                            }}
                          />
                        )}
                      </li>
                      <li>
                        <button
                          onClick={() => setSubMenuOpen(subMenuOpen === 'deleteDb' ? undefined : 'deleteDb')}
                          className={`${session.databases.length === 0 ? 'btn-disabled text-gray-300' : ''} ${
                            subMenuOpen === 'deleteDb' ? 'btn-active' : ''
                          }`}
                        >
                          Delete Database
                        </button>
                        {subMenuOpen === 'deleteDb' && (
                          <DatabaseMenu
                            onClick={(db) => {
                              if (session.currentDatabase === db) {
                                dispatch(updateCurrentDatabase(''));
                              }
                              api.DeleteDatabase(db).catch((e) => {
                                dispatch(addError(e.message));
                              });
                              setSubMenuOpen(undefined);
                              setMenuOpen(false);
                            }}
                          />
                        )}
                      </li>
                      <div className="menu-title">
                        <div className="absolute left-0 h-0.5 w-full bg-offwhite-300"></div>
                        <h3 className="text-xs mt-3">Version: {buildInfo}</h3>
                      </div>
                    </>
                  ) : (
                    <>
                      <div className="menu-title">
                        <h2>user: {auth.userID}</h2>
                        <h3 className="text-xs">session: {auth.sessionID}</h3>
                      </div>
                      <div>
                        <button
                          className="btn btn-ghost"
                          onClick={() => {
                            setMenuOpen(false);
                          }}
                        >
                          <span>Login</span>
                          {/* !TODO */}
                        </button>
                      </div>
                      <div className="menu-title">
                        <div className="absolute left-0 h-0.5 w-full bg-offwhite-300"></div>
                        <h3 className="text-xs mt-3">Version: {buildInfo}</h3>
                      </div>
                    </>
                  )}
                </ul>
              </>
            )}
          </div>
        </div>
      </div>
    </div>
  );
};
